@@ -1,7 +1,7 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2005 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2003-2006 aMule Team ( admin@amule.org / http://www.amule.org )
 // Copyright (c) 2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
@@ -20,23 +20,19 @@
 // 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA, 02111-1307, USA
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 //
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-#pragma implementation "ServerList.h"
-#endif
-
 #include "Types.h"
 
-#include <wx/defs.h>		// Needed before any other wx/*.h
+#include <wx/defs.h>			// Needed before any other wx/*.h
 #ifdef __WXMSW__
 	#include <wx/msw/winundef.h>
 #endif
 
 #include <wx/txtstrm.h>
 #include <wx/wfstream.h>
-#include <wx/filename.h>	// Needed for wxFileName
+#include <wx/filename.h>		// Needed for wxFileName
 #include <wx/url.h>			// Needed for wxURL
 #include <wx/tokenzr.h>
 
@@ -44,26 +40,31 @@
 #include "ListenSocket.h"		// Needed for CListenSocket
 #include "DownloadQueue.h"		// Needed for CDownloadQueue
 #include "ServerConnect.h"		// Needed for CServerConnect
-#include "Server.h"				// Needed for CServer and SRV_PR_*
+#include "Server.h"			// Needed for CServer and SRV_PR_*
 #include "OtherStructs.h"		// Needed for ServerMet_Struct
-#include "Packet.h"				// Needed for CInvalidPacket
 #include "OPCodes.h"			// Needed for MET_HEADER
-#include "SafeFile.h"			// Needed for CSafeFile
+#include "CFile.h"			// Needed for CFile
 #include "HTTPDownload.h"		// Needed for HTTPThread
-#include "Preferences.h"		// Needed for CPreferences
-#include "amule.h"				// Needed for theApp
-#include "GetTickCount.h"		// Neeed for GetTickCount
-#include "NetworkFunctions.h"	// Needed for StringIPtoUint32
-#include "Statistics.h"			// Needed for CStatistics
-#include "StringFunctions.h" // Needed for unicode2char 
+#include "Preferences.h"		// Needed for thePrefs
+#include "amule.h"			// Needed for theApp
+#include "GetTickCount.h"		// Needed for GetTickCount
+#include "NetworkFunctions.h"		// Needed for StringIPtoUint32
+#include "Statistics.h"			// Needed for theStats
+#include <common/StringFunctions.h>		// Needed for unicode2char 
+#include "Tag.h"			// Needed for CTag
+#include "Packet.h"			// Neeed for CPacket
 #include "Logger.h"
-#include "Format.h"
+#include <common/Format.h>
+#include "IPFilter.h"
+#include "FileFunctions.h"		// Needed for UnpackArchive
+
+#include <algorithm>			// Needed for std::find
+
 
 CServerList::CServerList()
 {
-	serverpos = 0;
-	statserverpos = 0;
-	delservercount = 0;
+	m_serverpos = m_servers.end();
+	m_statserverpos = m_servers.end();
 	m_nLastED2KServerLinkCheck = ::GetTickCount();
 }
 
@@ -93,14 +94,22 @@ bool CServerList::LoadServerMet(const wxString& strFile)
 {
 	AddLogLineM( false, CFormat( _("Loading server.met file: %s") ) % strFile );
 	
-	bool merge = !list.IsEmpty();
+	bool merge = !m_servers.empty();
 	
 	if ( !wxFileExists(strFile) ) {
 		AddLogLineM( false, _("Server.met file not found!") );
 		return false;
 	}
 
-	CSafeFile servermet( strFile,CFile::read );
+	// Try to unpack the file, might be an archive
+	const wxChar* mets[] = { wxT("server.met"), NULL };
+	// Try to unpack the file, might be an archive
+	if (UnpackArchive(strFile, mets).second != EFT_Met) {
+		AddLogLineM(true, CFormat(_("Failed to load server.met file '%s', unknown format encountered.")) % strFile);
+		return false;
+	}	
+
+	CFile servermet( strFile,CFile::read );
 	if ( !servermet.IsOpened() ){ 
 		AddLogLineM( false, _("Failed to open server.met!") );
 		return false;
@@ -113,8 +122,9 @@ bool CServerList::LoadServerMet(const wxString& strFile)
 		byte version = servermet.ReadUInt8();
 		
 		if (version != 0xE0 && version != MET_HEADER) {
-			AddLogLineM(false, wxString::Format(_("Invalid versiontag in server.met (0x%x , size %i)!"),version, sizeof(version)));
-			throw CInvalidPacket(wxT("Corrupted server.met"));
+			AddLogLineM(true, wxString::Format(_("Server.met file corrupt, found invalid versiontag: 0x%x, size %i"), version, sizeof(version)));
+			Notify_ServerThaw();
+			return false;
 		}
 
 		uint32 fservercount = servermet.ReadUInt32();
@@ -123,7 +133,7 @@ bool CServerList::LoadServerMet(const wxString& strFile)
 		uint32 iAddCount = 0;
 
 		for ( uint32 j = 0; j < fservercount; ++j ) {
-			sbuffer.ip			= servermet.ReadUInt32();
+			sbuffer.ip		= servermet.ReadUInt32();
 			sbuffer.port		= servermet.ReadUInt16();
 			sbuffer.tagcount	= servermet.ReadUInt32();
 			
@@ -134,7 +144,11 @@ bool CServerList::LoadServerMet(const wxString& strFile)
 				newserver->AddTagFromFile(&servermet);
 			}
 
-			if ( newserver->GetPreferences() > SRV_PR_HIGH ) {
+			// Server priorities are not in sorted order
+			// High = 1, Low = 2, Normal = 0, so we have to check 
+			// in a less logical fashion.
+			int priority = newserver->GetPreferences();
+			if (priority < SRV_PR_MIN || priority > SRV_PR_MAX) {
 				newserver->SetPreference(SRV_PR_NORMAL);
 			}
 			
@@ -167,8 +181,12 @@ bool CServerList::LoadServerMet(const wxString& strFile)
 		} else {
 			AddLogLineM(true, wxString::Format(_("%d servers added"), iAddCount));
 		}
-	} catch (const CInvalidPacket& e) {
-		AddLogLineM(true,_("Error: the file server.met is corrupted"));
+	} catch (const CInvalidPacket& err) {
+		AddLogLineM(true, wxT("Error: the file server.met is corrupted: ") + err.what());
+		Notify_ServerThaw();
+		return false;
+	} catch (const CSafeIOException& err) {
+		AddLogLineM(true, wxT("IO error while reading 'server.met': ") + err.what());
 		Notify_ServerThaw();
 		return false;
 	}
@@ -189,18 +207,22 @@ bool CServerList::AddServer(CServer* in_server, bool fromUser)
 		}
 
 		return false;
-	} else if ( !in_server->HasDynIP() ) {
-		if ( !IsGoodIP( in_server->GetIP(), thePrefs::FilterLanIPs() ) ) {
-			if ( fromUser ) {
-				AddLogLineM( true,
-					CFormat( _("Server not added: The IP of [%s:%d] is filtered or invalid.") )
-						% in_server->GetAddress()
-						% in_server->GetPort()
-				);
-			}
-		
-			return false;
+	} else if (
+				!in_server->HasDynIP() &&
+				(
+					!IsGoodIP( in_server->GetIP(), thePrefs::FilterLanIPs() ) ||
+					theApp.ipfilter->IsFiltered( in_server->GetIP(), true )
+				)
+	          ) {
+		if ( fromUser ) {
+			AddLogLineM( true,
+				CFormat( _("Server not added: The IP of [%s:%d] is filtered or invalid.") )
+					% in_server->GetAddress()
+					% in_server->GetPort()
+			);
 		}
+	
+		return false;
 	}
 	
 	CServer* test_server = GetServerByAddress(in_server->GetAddress(), in_server->GetPort());
@@ -219,8 +241,10 @@ bool CServerList::AddServer(CServer* in_server, bool fromUser)
 		
 		return false;
 	}
-	
-	list.AddTail(in_server);
+
+	theStats::AddServer();
+
+	m_servers.push_back(in_server);
 	NotifyObservers( EventType( EventType::INSERTED, in_server ) );
 
 	if ( fromUser ) {
@@ -239,35 +263,34 @@ bool CServerList::AddServer(CServer* in_server, bool fromUser)
 
 void CServerList::ServerStats()
 {
-	uint32 temp;
-	temp = (uint32)time(NULL);
-	
-	if(theApp.serverconnect->IsConnected() && list.GetCount() > 0) {
+
+	if(theApp.IsConnectedED2K() && m_servers.size() > 0) {
 		CServer* ping_server = GetNextStatServer();
 		CServer* test = ping_server;
 		if(!ping_server) {
 			return;
 		}
-	        while(ping_server->GetLastPinged() != 0 && (::GetTickCount() - ping_server->GetLastPinged()) < UDPSERVSTATREASKTIME) {
+
+		while(ping_server->GetLastPinged() != 0 && (::GetTickCount() - ping_server->GetLastPinged()) < UDPSERVSTATREASKTIME) {
 			ping_server = GetNextStatServer();
 			if(ping_server == test) {
 				return;
 			}
 		}
 		if(ping_server->GetFailedCount() >= thePrefs::GetDeadserverRetries() && thePrefs::DeadServer() && !ping_server->IsStaticMember()) {
-			Notify_ServerRemove(ping_server);
+			RemoveServer(ping_server);
 			return;
 		}
+				
 		CPacket* packet = new CPacket(OP_GLOBSERVSTATREQ, 4);
 		srand((unsigned)time(NULL));
-		uint32 time = 0x55AA0000 + (uint16)rand();
-		ping_server->SetChallenge(time);
-		packet->CopyUInt32ToDataBuffer(time);
+		uint32 challenge = 0x55AA0000 + (uint16)rand();
+		ping_server->SetChallenge(challenge);
+		packet->CopyUInt32ToDataBuffer(challenge);
 		ping_server->SetLastPinged(::GetTickCount());
-		//ping_server->SetLastPingedTime(temp);
 		ping_server->AddFailedCount();
 		Notify_ServerRefresh(ping_server);
-		theApp.statistics->AddUpDataOverheadServer(packet->GetPacketSize());
+		theStats::AddUpOverheadServer(packet->GetPacketSize());
 		theApp.serverconnect->SendUDPPacket(packet, ping_server, true);
 		
 		ping_server->SetLastDescPingedCount(false);
@@ -280,7 +303,7 @@ void CServerList::ServerStats()
 			uint32 uDescReqChallenge = ((uint32)randomness << 16) + INV_SERV_DESC_LEN; // 0xF0FF = an 'invalid' string length.
 			packet = new CPacket( OP_SERVER_DESC_REQ,4);
 			packet->CopyUInt32ToDataBuffer(uDescReqChallenge);
-			theApp.statistics->AddUpDataOverheadServer(packet->GetPacketSize());
+			theStats::AddUpOverheadServer(packet->GetPacketSize());
 			theApp.serverconnect->SendUDPPacket(packet, ping_server, true);
 		} else {
 			ping_server->SetLastDescPingedCount(true);
@@ -289,24 +312,30 @@ void CServerList::ServerStats()
 }
 
 
-void CServerList::RemoveServer(CServer* out_server)
+void CServerList::RemoveServer(CServer* in_server)
 {
-	if (out_server == theApp.serverconnect->GetCurrentServer()) {
+	if (in_server == theApp.serverconnect->GetCurrentServer()) {
 		theApp.ShowAlert(_("You are connected to the server you are trying to delete. please disconnect first."), _("Info"), wxOK);	
 	} else {
-	
-		POSITION pos = list.Find( out_server );
-		if ( pos != NULL ) {
-			if (theApp.downloadqueue->GetUDPServer() == out_server) {
+		CInternalList::iterator it = std::find(m_servers.begin(), m_servers.end(), in_server);
+		if ( it != m_servers.end() ) {
+			if (theApp.downloadqueue->GetUDPServer() == in_server) {
 				theApp.downloadqueue->SetUDPServer( 0 );
 			}	
 			
-			NotifyObservers( EventType( EventType::REMOVED, out_server ) );
+			NotifyObservers( EventType( EventType::REMOVED, in_server ) );
 		
-			list.RemoveAt(pos);
-			++delservercount;
-			delete out_server;
-			return;
+			if (m_serverpos == it) {
+				++m_serverpos;
+			}
+			if (m_statserverpos == it) {
+				++m_statserverpos;
+			}
+			m_servers.erase(it);
+			theStats::DeleteServer();
+			
+			Notify_ServerRemove(in_server);
+			delete in_server;
 		}
 	}
 }
@@ -316,18 +345,19 @@ void CServerList::RemoveAllServers()
 {
 	NotifyObservers( EventType( EventType::CLEARED ) );
 	
-	delservercount += list.GetSize();
+	theStats::DeleteAllServers();
 	// no connection, safely remove all servers
-	while ( !list.IsEmpty() ) {
-		delete list.GetTail();
-		list.RemoveTail();
+	while ( !m_servers.empty() ) {
+		delete m_servers.back();
+		m_servers.pop_back();
 	}
+	m_serverpos = m_servers.end();
+	m_statserverpos = m_servers.end();
 }
 
 
-void CServerList::GetStatus(uint32 &total, uint32 &failed, uint32 &user, uint32 &file, uint32 &tuser, uint32 &tfile,float &occ)
+void CServerList::GetStatus(uint32 &failed, uint32 &user, uint32 &file, uint32 &tuser, uint32 &tfile,float &occ)
 {
-	total = list.GetCount();
 	failed = 0;
 	user = 0;
 	file = 0;
@@ -337,9 +367,8 @@ void CServerList::GetStatus(uint32 &total, uint32 &failed, uint32 &user, uint32
 	uint32 maxusers=0;
 	uint32 tuserk = 0;
 
-	CServer* curr;
-	for (POSITION pos = list.GetHeadPosition(); pos !=0; ) {
-		curr = (CServer*)list.GetNext(pos);
+	for (CInternalList::const_iterator it = m_servers.begin(); it != m_servers.end(); ++it) {
+		const CServer* const curr = *it;
 		if( curr->GetFailedCount() ) {
 			++failed;
 		} else {
@@ -364,9 +393,8 @@ void CServerList::GetUserFileStatus(uint32 &user, uint32 &file)
 {
 	user = 0;
 	file = 0;
-	CServer* curr;
-	for (POSITION pos = list.GetHeadPosition(); pos !=0; ) {
-		curr = (CServer*)list.GetNext(pos);
+	for (CInternalList::const_iterator it = m_servers.begin(); it != m_servers.end(); ++it) {
+		const CServer* const curr = *it;
 		if( !curr->GetFailedCount() ) {
 			user += curr->GetUsers();
 			file += curr->GetFiles();
@@ -378,9 +406,9 @@ void CServerList::GetUserFileStatus(uint32 &user, uint32 &file)
 CServerList::~CServerList()
 {
 	SaveServerMet();
-	while ( !list.IsEmpty() ) {
-		delete list.GetTail();
-		list.RemoveTail();
+	while ( !m_servers.empty() ) {
+		delete m_servers.back();
+		m_servers.pop_back();
 	}
 }
 
@@ -419,7 +447,7 @@ void CServerList::LoadStaticServers( const wxString& filename )
 
 		
 		int priority = StrToLong( prio );
-		if ( priority < 0 || priority > 2 ) {
+		if (priority < SRV_PR_MIN || priority > SRV_PR_MAX) {
 			priority = SRV_PR_NORMAL;
 		}
 
@@ -454,80 +482,72 @@ void CServerList::LoadStaticServers( const wxString& filename )
 }
 
 
-void CServerList::Sort()
-{
-	POSITION pos1, pos2;
-	uint16 i = 0;
-	for( pos1 = list.GetHeadPosition(); (pos2 = pos1 ) != NULL;) {
-		list.GetNext(pos1);
-		CServer* cur_server = list.GetAt(pos2);
-		if (cur_server->GetPreferences()== SRV_PR_HIGH) {
-			list.AddHead(cur_server);
-			list.RemoveAt(pos2);
-		} else if (cur_server->GetPreferences() == SRV_PR_LOW) {
-			list.AddTail(cur_server);
-			list.RemoveAt(pos2);
-		}
-		++i;
-		if (i == list.GetCount()) {
-			break;
+struct ServerPriorityComparator {
+	// Return true iff lhs should strictly appear earlier in the list than rhs.
+	// In this case, we want higher priority servers to appear earlier.
+	bool operator()(const CServer* lhs, const CServer* rhs) {
+		wxASSERT
+			(
+			rhs->GetPreferences() == SRV_PR_LOW		||
+			rhs->GetPreferences() == SRV_PR_NORMAL	||
+			rhs->GetPreferences() == SRV_PR_HIGH
+			);
+		switch (lhs->GetPreferences()) {
+			case SRV_PR_LOW:
+				return false;
+			case SRV_PR_NORMAL:
+				return rhs->GetPreferences() == SRV_PR_LOW;
+			case SRV_PR_HIGH:
+				return rhs->GetPreferences() != SRV_PR_HIGH;
+			default:
+				wxASSERT(0);
+				return false;
 		}
 	}
+};
+
+void CServerList::Sort()
+{
+	m_servers.sort(ServerPriorityComparator());
+	// Once the list has been sorted, it doesn't really make sense to continue
+	// traversing the new order from the old position.  Plus, there's a bug in
+	// version of libstdc++ before gcc4 such that iterators that were equal to
+	// end() were left dangling.
+	m_serverpos = m_servers.begin();
+	m_statserverpos = m_servers.begin();
 }
 
 
 CServer* CServerList::GetNextServer()
 {
-	CServer* nextserver = 0;
-	uint32 i = 0;
-	if (serverpos>=((uint32)list.GetCount())) {
+	if (m_serverpos == m_servers.end()) {
 		return 0;
+	} else {
+		wxASSERT(*m_serverpos != NULL);
+		return *m_serverpos++;
 	}
-	while (!nextserver && i != (uint32)list.GetCount()) {
-		POSITION posIndex = list.FindIndex(serverpos);
-		if (posIndex == NULL) {	// check if search position is still valid (could be corrupted by server delete operation)
-			posIndex = list.GetHeadPosition();
-			serverpos= 0; // <<--9/27/02 zg
-		}
-
-		nextserver = list.GetAt(posIndex);
-		++serverpos;
-		++i;
-		// TODO: Add more option to filter bad server like min ping, min users etc
-		//if (nextserver->preferences = ?)
-		//	nextserver = 0;
-		//if (serverpos == list.GetCount()) return 0;//	serverpos = 0;
-	}
-	return nextserver;
 }
 
 
 CServer* CServerList::GetNextStatServer()
 {
-	CServer* nextserver = 0;
-	uint32 i = 0;
-	while (!nextserver && i != (uint32)list.GetCount()) {
-		POSITION posIndex = list.FindIndex(statserverpos);
-		if (posIndex == NULL) {	// check if search position is still valid (could be corrupted by server delete operation)
-			posIndex = list.GetHeadPosition();
-			statserverpos=0;
-		}
+	if (m_statserverpos == m_servers.end()) {
+		m_statserverpos = m_servers.begin();
+	}
 
-		nextserver = list.GetAt(posIndex);
-		++statserverpos;
-		++i;
-		if (statserverpos == (uint32)list.GetCount()) {
-			statserverpos = 0;
-		}
+	if (m_statserverpos == m_servers.end()) {
+		return 0;
+	} else {
+		wxASSERT(*m_statserverpos != NULL);
+		return *m_statserverpos++;
 	}
-	return nextserver;
 }
 
 
 CServer* CServerList::GetServerByAddress(const wxString& address, uint16 port)
 {
-	for (POSITION pos = list.GetHeadPosition();pos != 0;) {
-		CServer *s = list.GetNext(pos); // i_a: small speed optimization
+	for (CInternalList::const_iterator it = m_servers.begin(); it != m_servers.end(); ++it) {
+		CServer* const s = *it;
 		if (port == s->GetPort() && s->GetAddress() == address) {
 			return s;
 		}
@@ -537,8 +557,8 @@ CServer* CServerList::GetServerByAddress(const wxString& address, uint16 port)
 
 
 CServer* CServerList::GetServerByIP(uint32 nIP){
-	for (POSITION pos = list.GetHeadPosition();pos != 0;){
-        CServer* s = list.GetNext(pos);
+	for (CInternalList::const_iterator it = m_servers.begin(); it != m_servers.end(); ++it){
+        CServer* const s = *it;
 		if (s->GetIP() == nIP)
 			return s; 
 	}
@@ -547,8 +567,8 @@ CServer* CServerList::GetServerByIP(uint32 nIP){
 
 
 CServer* CServerList::GetServerByIP(uint32 nIP, uint16 nPort){
-	for (POSITION pos = list.GetHeadPosition();pos != 0;){
-        CServer* s = list.GetNext(pos);
+	for (CInternalList::const_iterator it = m_servers.begin(); it != m_servers.end(); ++it){
+        CServer* const s = *it;
 		if (s->GetIP() == nIP && s->GetPort() == nPort)
 			return s; 
 	}
@@ -560,91 +580,95 @@ bool CServerList::SaveServerMet()
 {
 	wxString newservermet = theApp.ConfigDir + wxT("server.met.new");
 	
-	CSafeFile servermet( newservermet, CFile::write );
+	CFile servermet( newservermet, CFile::write );
 	if (!servermet.IsOpened()) {
 		AddLogLineM(false,_("Failed to save server.met!"));
 		return false;
 	}
 
 
-	servermet.WriteUInt8(0xE0);
-	servermet.WriteUInt32( list.GetCount() );
+	try {
+		servermet.WriteUInt8(0xE0);
+		servermet.WriteUInt32( m_servers.size() );
 
-	
-	for ( POSITION pos = list.GetHeadPosition(); pos; ) {
-		CServer* server = list.GetNext( pos );
+		for ( CInternalList::const_iterator it = m_servers.begin(); it != m_servers.end(); ++it) {
+			const CServer* const server = *it;
 
-		uint16 tagcount = 12;
-		if ( !server->GetListName().IsEmpty() ) 			++tagcount;
-		if ( !server->GetDynIP().IsEmpty() )				++tagcount;
-		if ( !server->GetDescription().IsEmpty() )			++tagcount;
-		if ( server->GetConnPort() != server->GetPort() )	++tagcount;		
-		#if wxUSE_UNICODE
-		// For unicoded name, description, and dynip
-		if ( !server->GetListName().IsEmpty() ) {
-			++tagcount;
-		}
-		if ( !server->GetDynIP().IsEmpty() ) {
-			++tagcount;
-		}
-		if ( !server->GetDescription().IsEmpty() ) {
-			++tagcount;
-		}
-		if (!server->GetVersion().IsEmpty()){
-			++tagcount;
-		}
-		#endif
-		
-		
-		servermet.WriteUInt32(server->GetIP());
-		servermet.WriteUInt16(server->GetPort());
-		servermet.WriteUInt32(tagcount);
-					
-		if ( !server->GetListName().IsEmpty() ) {
+			uint16 tagcount = 12;
+			if ( !server->GetListName().IsEmpty() ) 			++tagcount;
+			if ( !server->GetDynIP().IsEmpty() )				++tagcount;
+			if ( !server->GetDescription().IsEmpty() )			++tagcount;
+			if ( server->GetConnPort() != server->GetPort() )	++tagcount;		
 			#if wxUSE_UNICODE
-			// This is BOM to keep eMule compatibility
-			CTag( ST_SERVERNAME,	server->GetListName()		).WriteTagToFile( &servermet,  utf8strOptBOM);
+			// For unicoded name, description, and dynip
+			if ( !server->GetListName().IsEmpty() ) {
+				++tagcount;
+			}
+			if ( !server->GetDynIP().IsEmpty() ) {
+				++tagcount;
+			}
+			if ( !server->GetDescription().IsEmpty() ) {
+				++tagcount;
+			}
+			if (!server->GetVersion().IsEmpty()){
+				++tagcount;
+			}
 			#endif
-			CTag( ST_SERVERNAME,	server->GetListName()		).WriteTagToFile( &servermet );
+			
+			
+			servermet.WriteUInt32(server->GetIP());
+			servermet.WriteUInt16(server->GetPort());
+			servermet.WriteUInt32(tagcount);
+						
+			if ( !server->GetListName().IsEmpty() ) {
+				#if wxUSE_UNICODE
+				// This is BOM to keep eMule compatibility
+				CTag( ST_SERVERNAME,	server->GetListName()		).WriteTagToFile( &servermet,  utf8strOptBOM);
+				#endif
+				CTag( ST_SERVERNAME,	server->GetListName()		).WriteTagToFile( &servermet );
+			}
+			
+			if ( !server->GetDynIP().IsEmpty() ) {
+				#if wxUSE_UNICODE
+				// This is BOM to keep eMule compatibility
+				CTag( ST_DYNIP,			server->GetDynIP()			).WriteTagToFile( &servermet, utf8strOptBOM );
+				#endif			
+				CTag( ST_DYNIP,			server->GetDynIP()			).WriteTagToFile( &servermet );
+			}
+			
+			if ( !server->GetDescription().IsEmpty() ) {
+				#if wxUSE_UNICODE
+				// This is BOM to keep eMule compatibility
+				CTag( ST_DESCRIPTION,	server->GetDescription()	).WriteTagToFile( &servermet, utf8strOptBOM );
+				#endif			
+				CTag( ST_DESCRIPTION,	server->GetDescription()	).WriteTagToFile( &servermet );
+			}
+			
+			if ( server->GetConnPort() != server->GetPort() ) {
+				CTag( ST_AUXPORTSLIST,	server->GetAuxPortsList()	).WriteTagToFile( &servermet );
+			}
+			
+			CTag( ST_FAIL,			server->GetFailedCount()	).WriteTagToFile( &servermet );
+			CTag( ST_PREFERENCE,	server->GetPreferences()	).WriteTagToFile( &servermet );
+			CTag( "users",			server->GetUsers()			).WriteTagToFile( &servermet );
+			CTag( "files",			server->GetFiles()			).WriteTagToFile( &servermet );
+			CTag( ST_PING,			server->GetPing()			).WriteTagToFile( &servermet );
+			CTag( ST_LASTPING,		server->GetLastPinged()		).WriteTagToFile( &servermet );
+			CTag( ST_MAXUSERS,		server->GetMaxUsers()		).WriteTagToFile( &servermet );
+			CTag( ST_SOFTFILES,		server->GetSoftFiles()		).WriteTagToFile( &servermet );
+			CTag( ST_HARDFILES,		server->GetHardFiles()		).WriteTagToFile( &servermet );
+			if (!server->GetVersion().IsEmpty()){
+				#if wxUSE_UNICODE			
+				CTag( ST_VERSION,		server->GetVersion()		).WriteTagToFile( &servermet, utf8strOptBOM );
+				#endif
+				CTag( ST_VERSION,		server->GetVersion()		).WriteTagToFile( &servermet );
+			}
+			CTag( ST_UDPFLAGS,		server->GetUDPFlags()		).WriteTagToFile( &servermet );
+			CTag( ST_LOWIDUSERS,	server->GetLowIDUsers()		).WriteTagToFile( &servermet );
 		}
-		
-		if ( !server->GetDynIP().IsEmpty() ) {
-			#if wxUSE_UNICODE
-			// This is BOM to keep eMule compatibility
-			CTag( ST_DYNIP,			server->GetDynIP()			).WriteTagToFile( &servermet, utf8strOptBOM );
-			#endif			
-			CTag( ST_DYNIP,			server->GetDynIP()			).WriteTagToFile( &servermet );
-		}
-		
-		if ( !server->GetDescription().IsEmpty() ) {
-			#if wxUSE_UNICODE
-			// This is BOM to keep eMule compatibility
-			CTag( ST_DESCRIPTION,	server->GetDescription()	).WriteTagToFile( &servermet, utf8strOptBOM );
-			#endif			
-			CTag( ST_DESCRIPTION,	server->GetDescription()	).WriteTagToFile( &servermet );
-		}
-		
-		if ( server->GetConnPort() != server->GetPort() ) {
-			CTag( ST_AUXPORTSLIST,	server->GetAuxPortsList()	).WriteTagToFile( &servermet );
-		}
-		
-		CTag( ST_FAIL,			server->GetFailedCount()	).WriteTagToFile( &servermet );
-		CTag( ST_PREFERENCE,	server->GetPreferences()	).WriteTagToFile( &servermet );
-		CTag( "users",			server->GetUsers()			).WriteTagToFile( &servermet );
-		CTag( "files",			server->GetFiles()			).WriteTagToFile( &servermet );
-		CTag( ST_PING,			server->GetPing()			).WriteTagToFile( &servermet );
-		CTag( ST_LASTPING,		server->GetLastPinged()		).WriteTagToFile( &servermet );
-		CTag( ST_MAXUSERS,		server->GetMaxUsers()		).WriteTagToFile( &servermet );
-		CTag( ST_SOFTFILES,		server->GetSoftFiles()		).WriteTagToFile( &servermet );
-		CTag( ST_HARDFILES,		server->GetHardFiles()		).WriteTagToFile( &servermet );
-		if (!server->GetVersion().IsEmpty()){
-			#if wxUSE_UNICODE			
-			CTag( ST_VERSION,		server->GetVersion()		).WriteTagToFile( &servermet, utf8strOptBOM );
-			#endif
-			CTag( ST_VERSION,		server->GetVersion()		).WriteTagToFile( &servermet );
-		}
-		CTag( ST_UDPFLAGS,		server->GetUDPFlags()		).WriteTagToFile( &servermet );
-		CTag( ST_LOWIDUSERS,	server->GetLowIDUsers()		).WriteTagToFile( &servermet );
+	} catch (const CIOFailureException& e) {
+		AddLogLineM(true, wxT("IO failure while writing 'server.met': ") + e.what());
+		return false;
 	}
 	
 	servermet.Close();
@@ -668,10 +692,10 @@ bool CServerList::SaveServerMet()
 void CServerList::RemoveDeadServers()
 {
 	if ( thePrefs::DeadServer() ) {
-		for ( POSITION pos = list.GetHeadPosition(); pos != NULL; ) {
-			CServer* cur_server = list.GetNext( pos );
-			if ( cur_server->GetFailedCount() > thePrefs::GetDeadserverRetries() && !cur_server->IsStaticMember()) {
-				RemoveServer(cur_server);
+		for ( CInternalList::const_iterator it = m_servers.begin(); it != m_servers.end(); ) {
+			CServer* server = *it++;
+			if ( server->GetFailedCount() > thePrefs::GetDeadserverRetries() && !server->IsStaticMember()) {
+				RemoveServer(server);
 			}
 		}
 	}
@@ -690,7 +714,8 @@ void CServerList::UpdateServerMetFromURL(const wxString& strURL)
 	downloader->Run();
 }
 
-void CServerList::DownloadFinished(uint32 result) {
+void CServerList::DownloadFinished(uint32 result) 
+{
 	if(result==1) {
 		wxString strTempFilename(theApp.ConfigDir + wxT("server.met.download"));
 		// curl succeeded. proceed with server.met loading
@@ -704,7 +729,8 @@ void CServerList::DownloadFinished(uint32 result) {
 }
 
 
-void CServerList::AutoUpdate() {
+void CServerList::AutoUpdate() 
+{
 	
 	uint8 url_count = theApp.glob_prefs->adresses_list.GetCount();
 	
@@ -742,7 +768,9 @@ void CServerList::AutoUpdate() {
 }
 
 
-void CServerList::AutoDownloadFinished(uint32 result) {
+void CServerList::AutoDownloadFinished(uint32 result) 
+{
+	
 	if(result==1) {
 		wxString strTempFilename(theApp.ConfigDir + wxT("server_auto.met"));
 		// curl succeeded. proceed with server.met loading
@@ -751,7 +779,7 @@ void CServerList::AutoDownloadFinished(uint32 result) {
 		// So, file is loaded and merged, and also saved
 		wxRemoveFile(strTempFilename);
 	} else {
-		AddLogLineM(true, wxString( _("Failed to download the server list from ") ) + URLUpdate);
+		AddLogLineM(true, CFormat(_("Failed to download the server list from %s") ) % URLUpdate);
 	}
 	
 	++current_url_index;
@@ -770,11 +798,46 @@ void CServerList::ObserverAdded( ObserverType* o )
 	CObservableQueue<CServer*>::ObserverAdded( o );
 
 	EventType::ValueList ilist;
-	ilist.reserve( list.GetSize() );
-	
-	for ( POSITION pos = list.GetHeadPosition(); pos; ) {
-		ilist.push_back( list.GetNext(pos) );
-	}
+	ilist.reserve( m_servers.size() );
+	ilist.assign( m_servers.begin(), m_servers.end() );
 
 	NotifyObservers( EventType( EventType::INITIAL, &ilist ), o );
 }
+
+
+uint32 CServerList::GetAvgFile() const
+{
+	//Since there is no real way to know how many files are in the kad network,
+	//I figure to try to use the ED2K network stats to find how many files the
+	//average user shares..
+	uint32 totaluser = 0;
+	uint32 totalfile = 0;
+	for (CInternalList::const_iterator it = m_servers.begin(); it != m_servers.end(); ++it){
+		const CServer* const curr = *it;
+		//If this server has reported Users/Files and doesn't limit it's files too much
+		//use this in the calculation..
+		if( curr->GetUsers() && curr->GetFiles() && curr->GetSoftFiles() > 1000 ) {
+			totaluser += curr->GetUsers();
+			totalfile += curr->GetFiles();
+		}
+	}
+	//If the user count is a little low, don't send back a average..
+	//I added 50 to the count as many servers do not allow a large amount of files to be shared..
+	//Therefore the estimate here will be lower then the actual.
+	//I would love to add a way for the client to send some statistics back so we could see the real
+	//values here..
+	if ( totaluser > 500000 ) {
+		return (totalfile/totaluser)+50;
+	} else {
+		return 0;
+	}
+}
+
+
+std::vector<const CServer*> CServerList::CopySnapshot() const
+{
+	std::vector<const CServer*> result;
+	result.reserve(m_servers.size());
+	result.assign(m_servers.begin(), m_servers.end());
+	return result;
+}
