@@ -1,7 +1,7 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2005 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2003-2006 aMule Team ( admin@amule.org / http://www.amule.org )
 // Copyright (c) 2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
@@ -20,16 +20,12 @@
 // 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA, 02111-1307, USA
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 //
 
 #ifndef SERVERLISTCTRL_H
 #define SERVERLISTCTRL_H
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-#pragma interface "ServerListCtrl.h"
-#endif
-
 #include "MuleListCtrl.h"	// Needed for CMuleListCtrl
 
 #define	COLUMN_SERVER_NAME	0
@@ -90,21 +86,16 @@ public:
 	void	AddServer( CServer* toadd );
 	
 	/**
-	 * Removes a server from the list.
-	 *
-	 * @param server The server to be removed.
-	 *
-	 * Please note that this function ALSO removes the server from the actual
-	 * server list, not just from the displayed one!
+	 * Removes a server from the displayed list.
 	 */
-	void	RemoveServer( const CServer* server, bool ask_static = false);
+	void	RemoveServer(CServer* server);
 	
 	/**
 	 * Removes all servers with the specified state. 
 	 *
 	 * @param state All items with this state will be removed, default being all.
 	 */
-	void	RemoveAllServers( int state = wxLIST_STATE_DONTCARE, bool ask_static = false );
+	void	RemoveAllServers(int state = wxLIST_STATE_DONTCARE);
 	
 	
 	/**
