@@ -317,7 +317,7 @@ void CamuleRemoteGuiApp::Startup() {
 	// bugfix - do this before creating the uploadqueue
 	downloadqueue = new CDownQueueRem(connect);
 	uploadqueue = new CUpQueueRem(connect);
-	ipfilter = new CIPFilterRem();
+	ipfilter = new CIPFilterRem(connect);
 
 	// Parse cmdline arguments.
 	wxCmdLineParser cmdline(wxApp::argc, wxApp::argv);
@@ -796,16 +796,23 @@ void CServerListRem::ReloadControl()
 	}
 }
 
+
+CIPFilterRem::CIPFilterRem(CRemoteConnect* conn)
+{
+	m_conn = conn;
+}
+
+
 void CIPFilterRem::Reload()
 {
 	CECPacket req(EC_OP_IPFILTER_RELOAD);
 	m_conn->SendPacket(&req);
 }
 
-void CIPFilterRem::Update(wxString /*url*/)
+void CIPFilterRem::Update(wxString WXUNUSED(url))
 {
 	// FIXME: add command
-	wxASSERT(0);
+	//wxASSERT(0);
 }
 
 /*
@@ -813,6 +820,7 @@ void CIPFilterRem::Update(wxString /*url*/)
  */
 CSharedFilesRem::CSharedFilesRem(CRemoteConnect *conn) : CRemoteContainer<CKnownFile, CMD4Hash, CEC_SharedFile_Tag>(conn, true)
 {
+	m_rename_file = NULL;
 }
 
 void CSharedFilesRem::Reload(bool, bool)
@@ -850,7 +858,7 @@ void CSharedFilesRem::HandlePacket(const CECPacket *packet)
 {
 	if ( m_rename_file && (packet->GetOpCode() == EC_OP_NOOP) ) {
 		m_rename_file->SetFileName(m_new_name);
-		m_rename_file = 0;
+		m_rename_file = NULL;
 	} else if ( packet->GetOpCode() != EC_OP_FAILED ) {
 		CRemoteContainer<CKnownFile, CMD4Hash, CEC_SharedFile_Tag>::HandlePacket(packet);
 	}
@@ -1195,7 +1203,7 @@ void CDownQueueRem::ProcessItemUpdate(CEC_PartFile_Tag *tag, CPartFile *file)
 		file->m_notCurrentSources = tag->SourceNotCurrCount();
 		file->m_source_count = tag->SourceCount();
 		file->m_a4af_source_count = tag->SourceCountA4AF();
-	    file->status = tag->FileStatus();
+		file->status = tag->FileStatus();
 	
 		file->lastseencomplete = tag->LastSeenComplete();
 		
