@@ -1,8 +1,8 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2004-2005 Angel Vidal (Kry) ( kry@amule.org )
-// Copyright (c) 2004-2005 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2004-2006 Angel Vidal (Kry) ( kry@amule.org )
+// Copyright (c) 2004-2006 aMule Team ( admin@amule.org / http://www.amule.org )
 // Copyright (c) 2003 Barry Dunne (http://www.emule-project.net)
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
@@ -21,7 +21,7 @@
 // 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA, 02111-1307, USA
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 //
 
 // Note To Mods //
@@ -50,80 +50,28 @@ namespace Kademlia {
 
 class CUInt128;
 
-class CTagNameString : protected wxString
-{
-public:
-	CTagNameString()
-	{
-	}
-
-	CTagNameString(const wxString& psz)
-		: wxString(psz)
-	{
-	}
-
-	CTagNameString(wxChar* psz, int len)
-		: wxString(psz, len)
-	{
-	}
-
-	// A tag name may include character values >= 0xD0 and therefor also >= 0xF0. to prevent those
-	// characters be interpreted as multi byte character sequences we have to ensure that a binary
-	// string compare is performed.
-	int Compare(const wxString& psz) const throw()
-	{
-		// Do a binary string compare. (independant from any codepage and/or LC_CTYPE setting.)
-		return strcmp((char*)c_str(), (char*)psz.c_str());
-	}
-
-	int CompareNoCase(const wxString& psz) const throw()
-	{
-		return CmpNoCase(psz);
-	}
-
-	CTagNameString& operator=(const wxString& pszSrc)
-	{
-		wxString::operator=(pszSrc);
-		return *this;
-	}
-	
-	const wxString	GetString() const {
-		return (wxString) GetData();
-	}
-
-#if !wxUSE_STL	
-	wxChar* GetBuffer(int nMinBufferLength)
-	{
-		return wxString::GetWriteBuf(nMinBufferLength);
-	}
-
-	void ReleaseBuffer() {
-		wxString::UngetWriteBuf();
-	}
-#else
-	#warning Need a replacement for wxString::GetWriteBuf()/UngetWriteBuf() !
-#endif
-
-	int GetLength() const throw()
-	{
-		return wxString::Len();
-	}
-};
-
-
 #define CTagValueString wxString
 
 class CTag
 {
 public:
 	byte	m_type;
-	CTagNameString m_name;
+	wxString m_name;
 
 	CTag(byte type, const wxString& name)
 		: m_name(name)
 	{
 		m_type = type;
 	}
+	
+	#if wxUSE_UNICODE
+	CTag(byte type, const char* name)
+		: m_name(wxString::FromAscii(name))
+	{
+		m_type = type;
+	}	
+	#endif
+	
 	virtual ~CTag() {}
 
 	bool IsStr()  const { return m_type == TAGTYPE_STRING; }
@@ -152,6 +100,11 @@ public:
 	CTagUnk(byte type, const wxString& name)
 		: CTag(type, name)
 	{ }
+	#if wxUSE_UNICODE
+	CTagUnk(byte type, const char* name)
+		: CTag(type, wxString::FromAscii(name))
+	{ }	
+	#endif
 };
 
 
@@ -162,6 +115,13 @@ public:
 		: CTag(TAGTYPE_STRING, name)
 		, m_value(value)
 	{ }
+	
+	#if wxUSE_UNICODE
+	CTagStr(const char* name, const wxString& value)
+		: CTag(TAGTYPE_STRING, wxString::FromAscii(name))
+		, m_value(value)
+	{ }
+	#endif
 
 	virtual CTagValueString GetStr() const { return m_value; }
 
@@ -177,6 +137,14 @@ public:
 		: CTag(0xFE, name)
 		, m_value(value)
 	{ }
+	
+	#if wxUSE_UNICODE
+	CTagUInt(const char* name, uint32 value)
+		: CTag(0xFE, wxString::FromAscii(name))
+		, m_value(value)
+	{ }
+	#endif
+
 
 	virtual uint32 GetInt() const { return m_value; }
 
@@ -192,6 +160,13 @@ public:
 		: CTag(TAGTYPE_UINT32, name)
 		, m_value(value)
 	{ }
+	
+	#if wxUSE_UNICODE
+	CTagUInt32(const char* name, uint32 value)
+		: CTag(TAGTYPE_UINT32, wxString::FromAscii(name))
+		, m_value(value)
+	{ }
+	#endif
 
 	virtual uint32 GetInt() const { return m_value; }
 
@@ -208,6 +183,13 @@ public:
 		, m_value(value)
 	{ }
 
+	#if wxUSE_UNICODE
+	CTagFloat(const char* name, float value)
+		: CTag(TAGTYPE_FLOAT32, wxString::FromAscii(name))
+		, m_value(value)
+	{ }
+	#endif
+	
 	virtual float GetFloat() const { return m_value; }
 
 protected:
@@ -222,6 +204,13 @@ public:
 		: CTag(TAGTYPE_BOOL, name)
 		, m_value(value)
 	{ }
+	
+	#ifdef wxUSE_UNICODE
+	CTagBool(const char* name, bool value)
+		: CTag(TAGTYPE_BOOL, wxString::FromAscii(name))
+		, m_value(value)
+	{ }
+	#endif	
 
 	virtual bool GetBool() const { return m_value; }
 
@@ -233,10 +222,18 @@ protected:
 class CTagUInt16 : public CTag
 {
 public:
+
 	CTagUInt16(const wxString& name, uint16 value)
 		: CTag(TAGTYPE_UINT16, name)
 		, m_value(value)
 	{ }
+	
+	#if wxUSE_UNICODE
+	CTagUInt16(const char* name, uint16 value)
+		: CTag(TAGTYPE_UINT16, wxString::FromAscii(name))
+		, m_value(value)
+	{ }
+	#endif
 
 	virtual uint32 GetInt() const { return m_value; }
 
@@ -253,6 +250,13 @@ public:
 		, m_value(value)
 	{ }
 
+	#ifdef wxUSE_UNICODE
+	CTagUInt8(const char* name, uint8 value)
+		: CTag(TAGTYPE_UINT8, wxString::FromAscii(name))
+		, m_value(value)
+	{ }
+	#endif
+
 	virtual uint32 GetInt() const { return m_value; }
 
 protected:
@@ -270,6 +274,16 @@ public:
 		memcpy(m_value, value, nSize);
 		m_size = nSize;
 	}
+	
+	#if wxUSE_UNICODE
+	CTagBsob(const char* name, const byte* value, uint8 nSize)
+		: CTag(TAGTYPE_BSOB, wxString::FromAscii(name))
+	{
+		m_value = new byte[nSize];
+		memcpy(m_value, value, nSize);
+		m_size = nSize;
+	}	
+	#endif
 
 	~CTagBsob()
 	{
@@ -292,8 +306,17 @@ public:
 		: CTag(TAGTYPE_HASH, name)
 	{ 
 		m_value = new byte[16];
-		otherfunctions::md4cpy(m_value, value);
+		md4cpy(m_value, value);
 	}
+	
+	#if wxUSE_UNICODE
+	CTagHash(const char* name, const byte* value) 
+		: CTag(TAGTYPE_HASH, wxString::FromAscii(name))
+	{ 
+		m_value = new byte[16];
+		md4cpy(m_value, value);
+	}
+	#endif	
 
 	~CTagHash()
 	{
