@@ -24,13 +24,9 @@
 /// You should have received a copy of the GNU General Public License
 /// along with this program; if not, write to the
 /// Free Software Foundation, Inc.,
-/// 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+/// 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
-#ifdef __GNUG__
-#pragma implementation "wxcasprint.h"
-#endif
-
 // For compilers that support precompilation, includes "wx/wx.h"
 #include "wx/wxprec.h"
 
@@ -131,5 +127,5 @@ WxCasPrint::DrawPageOne ( wxDC * dc )
 	dc->SetDeviceOrigin ( ( wxCoord ) posX, ( wxCoord ) posY );
 
 	// Draw image
-	dc->DrawBitmap ( wxBitmap( statImage ), 0, 0, FALSE );
+	dc->DrawBitmap ( wxBitmap( *statImage ), 0, 0, FALSE );
 }
