@@ -265,7 +265,7 @@ long CWebServerBase::GetWSPrefs(void)
 	// we have selected only the webserver preferences
 	const CECTag *wsprefs = reply->GetTagByIndexSafe(0);
 	const CECTag *tag = wsprefs->GetTagByName(EC_TAG_WEBSERVER_PORT);
-	long wsport = tag ? tag->GetInt16Data() : -1;
+	long int wsport = tag ? (long int)tag->GetInt16Data() : -1;
 
 	if (webInterface->m_LoadSettingsFromAmule) {
 		webInterface->m_AdminPass = wsprefs->GetTagByNameSafe(EC_TAG_PASSWD_HASH)->GetMD4Data();
@@ -294,19 +294,29 @@ long CWebServerBase::GetWSPrefs(void)
 	return wsport;
 }
 
-void CWebServerBase::ProcessImgFileReq(ThreadData Data)
+void CScriptWebServer::ProcessImgFileReq(ThreadData Data)
 {
 	webInterface->DebugShow(wxT("**** imgrequest: ") + Data.sURL + wxT("\n"));
 
-	CAnyImage *img = m_ImageLib.GetImage(Data.sURL);
-	if ( img ) {
-		int img_size;
+	const CSession* session = CheckLoggedin(Data);
+
+	// To prevent access to non-template images, we disallow use of paths in filenames.
+	wxString imgName = wxFileName::GetPathSeparator() + wxFileName(Data.sURL).GetFullName();
+	CAnyImage *img = m_ImageLib.GetImage(imgName);
+	
+	// Only static images are available to visitors, in order to prevent
+	// information leakage, but still allowing images on the login page.
+	if (session->m_loggedin or dynamic_cast<CFileImage*>(img)) {
+		int img_size = 0;
 		unsigned char* img_data = img->RequestData(img_size);
 		// This unicode2char is ok.
 		Data.pSocket->SendContent(unicode2char(img->GetHTTP()), img_data, img_size);
+	} else if (not session->m_loggedin) {
+		webInterface->DebugShow(wxT("**** imgrequest: failed, not logged in\n"));
+		ProcessURL(Data);
 	} else {
 		webInterface->DebugShow(wxT("**** imgrequest: failed\n"));
-	}	
+	}
 }
 
 // send EC request and discard output
@@ -342,7 +352,9 @@ void CWebServerBase::Send_Discard_V2_Request(CECPacket *request)
 void CWebServerBase::Send_SharedFile_Cmd(wxString file_hash, wxString cmd, uint32 opt_arg)
 {
 	CECPacket *ec_cmd = 0;
-	const CMD4Hash fileHash(file_hash);
+	CMD4Hash fileHash;
+	wxCHECK2(fileHash.Decode(file_hash), /* Do nothing. */ );
+	
 	CECTag hashtag(EC_TAG_KNOWNFILE, fileHash);
 	if (cmd == wxT("prio")) {
 		ec_cmd = new CECPacket(EC_OP_SHARED_SET_PRIO);
@@ -379,7 +391,9 @@ void CWebServerBase::Send_ReloadSharedFile_Cmd()
 void CWebServerBase::Send_DownloadFile_Cmd(wxString file_hash, wxString cmd, uint32 opt_arg)
 {
 	CECPacket *ec_cmd = 0;
-	const CMD4Hash fileHash(file_hash);
+	CMD4Hash fileHash;
+	wxCHECK2(fileHash.Decode(file_hash), /* Do nothing. */ );
+	
 	CECTag hashtag(EC_TAG_PARTFILE, fileHash);
 	if (cmd == wxT("pause")) {
 		ec_cmd = new CECPacket(EC_OP_PARTFILE_PAUSE);
@@ -415,8 +429,11 @@ void CWebServerBase::Send_DownloadFile_Cmd(wxString file_hash, wxString cmd, uin
 
 void CWebServerBase::Send_DownloadSearchFile_Cmd(wxString file_hash, uint8 cat)
 {
+	CMD4Hash fileHash;
+	wxCHECK2(fileHash.Decode(file_hash), /* Do nothing. */ );
+	
 	CECPacket ec_cmd(EC_OP_DOWNLOAD_SEARCH_RESULT);
-	CECTag link_tag(EC_TAG_KNOWNFILE, CMD4Hash(file_hash));
+	CECTag link_tag(EC_TAG_KNOWNFILE, fileHash);
 
 	link_tag.AddTag(CECTag(EC_TAG_PARTFILE_CAT, cat));
 	ec_cmd.AddTag(link_tag);
@@ -1671,7 +1688,8 @@ CScriptWebServer::CScriptWebServer(CamulewebApp *webApp, const wxString& templat
 	} else if ( ::wxFileExists(wxFileName(m_wwwroot, wxT("index.php")).GetFullPath()) ) {
 		m_index = wxT("index.php");
 	} else {
-		webInterface->Show(_("Index file not found: bad template\n"));
+		webInterface->Show(_("Index file not found: ") + 
+			wxFileName(m_wwwroot, wxT("index.{html,php}")).GetFullPath() + wxT("\n"));
 	}
 }
 
@@ -1798,6 +1816,7 @@ CSession *CScriptWebServer::CheckLoggedin(ThreadData &Data)
 	return session;
 }
 
+
 void CScriptWebServer::ProcessURL(ThreadData Data)
 {
 	wxMutexLocker lock(*m_mutexChildren);
@@ -1805,7 +1824,8 @@ void CScriptWebServer::ProcessURL(ThreadData Data)
 	long httpOutLen;
 	char *httpOut = 0;
 	
-	wxString filename = Data.parsedURL.File();
+	// Strip out any path-component to prevent information leakage.
+	wxString filename = wxFileName(Data.parsedURL.File()).GetFullName();
 
 	Print(_("Processing request [original]: ") + filename + wxT("\n"));
 
@@ -1815,22 +1835,32 @@ void CScriptWebServer::ProcessURL(ThreadData Data)
 
 	CSession *session = CheckLoggedin(Data);
 
+	session->m_vars["login_error"] = "";
 	if ( !session->m_loggedin ) {
-		filename = wxT("login.html");
+		filename = wxT("login.php");
+		
 		wxString PwStr(Data.parsedURL.Param(wxT("pass")));
-		if ( PwStr.Length() ) {
+		if (webInterface->m_AdminPass.IsEmpty() and webInterface->m_GuestPass.IsEmpty()) {
+			session->m_vars["login_error"] = "No passwords specified, login impossible!";
+		} else if ( PwStr.Length() ) {
 			Print(_("Checking password\n"));
-			CMD4Hash PwHash(MD5Sum(PwStr).GetHash());
-			
 			session->m_loggedin = false;
-			if ( PwHash == webInterface->m_AdminPass ) {
+			
+			CMD4Hash PwHash;
+			if (not PwHash.Decode(MD5Sum(PwStr).GetHash())) {
+				Print(_("Password hash invalid\n"));
+				session->m_vars["login_error"] = "Invalid password hash, please report!";
+			} else if ( PwHash == webInterface->m_AdminPass ) {
 				session->m_loggedin = true;
 				// m_vars is map<string, string> - so _() will not work here !
 				session->m_vars["guest_login"] = "0";
 			} else if ( PwHash == webInterface->m_GuestPass ) {
 				session->m_loggedin = true;
 				session->m_vars["guest_login"] = "1";
+			} else {
+				session->m_vars["login_error"] = "Password incorrect, please try again.";
 			}
+			
 			if ( session->m_loggedin ) {
 				filename = m_index;
 				Print(_("Password ok\n"));
@@ -1845,7 +1875,7 @@ void CScriptWebServer::ProcessURL(ThreadData Data)
 		// if logged in, but requesting login page again,
 		// means logout command
 		//
-		if ( filename == wxT("login.html") ) {
+		if ( filename == wxT("login.php") ) {
 			Print(_("Logout requested\n"));
 			session->m_loggedin = false;
 		}
